@@ -1,7 +1,8 @@
 #' @param fun a function or 'predict()'-like function that returns a simple numeric or mean and standard error: list(mean=...,se=...).
 #' @param vectorized is fun vectorized?
-#' @param dim input variables dimension of the model or function.
 #' @param col_fading_interval an optional factor of alpha (color channel) fading used to plot function output intervals (if any).
+#' @param col_interval color to display interval width.
+#' @param add_fading an optional factor of alpha (color channel) fading used to plot when add=TRUE.
 #' @template filledcontourview-doc
 #' @rdname filledcontourview
 #' @method filledcontourview function
@@ -16,7 +17,7 @@
 #' model <- lm(y ~ x1 + x2)
 #'
 #' filledcontourview(function(x) sum(x),
-#'                      dim=2, Xlim=cbind(range(x1),range(x2)), col='black')
+#'                      Xlim=cbind(range(x1),range(x2)), col='black')
 #' points(x1,x2)
 #'
 #' filledcontourview(function(x) {
@@ -27,103 +28,67 @@
 #'                   Xlim=cbind(range(x1),range(x2)), add=TRUE)
 #'
 filledcontourview.function <- function(fun, vectorized=FALSE,
-                                dim = NULL,
                              center = NULL,
                              lty_center = 2,
                              col_center = "black",
                              axis = NULL,
                              npoints = 21,
                              levels = 10,
-                             lty_levels = 1,
-                             col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels-1) else col.levels("blue",levels-1),
+                             lty_levels = 0,
+                             col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                              col = NULL,
+                             col_interval = 'white',
                              col_fading_interval = 0.5,
                              mfrow = NULL,
                              Xlab = NULL, ylab = NULL,
-                             Xlim = NULL,
-                             title = NULL,
+                             Xlim = if (!add) matrix(c(0,1),2,2) else NULL, ylim=NULL,
+                             title = NULL, title_sep = " | ",
                              add = FALSE,
+                             add_fading = 0.5,
                              ...) {
-    if (!is.null(dim)) {
-        D <- dim
-        if (is.null(center))
-            if (D != 2) stop("Section center in 'section' required for >2-D model.")
-    } else {
-        if (is.null(center))
-            stop("dim or center must be specified.")
-        else
-            D <- length(center)
-    }
-
-    if (length(levels)==1) {
-        levels = pretty(range(unlist(EvalInterval.function(fun,Xlim,vectorized,D)),na.rm=TRUE), levels)
-        if (length(col_levels) != length(levels)-1)
-            col_levels = col.levels(col_levels,levels)
-    }
+    # setup for center/Xlim/axis/mfrow
+    view = getView(2, add, center, Xlim, ylim, axis, mfrow)
+    D = view$D
+    center = view$center
+    Xlim = view$Xlim
+    ylim = view$ylim
+    axis = view$axis
+    mfrow = view$mfrow
 
     if (D == 1) stop("for a model with dim 1, use 'sectionview'")
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        ## added by YD for the vector case
-        axis <- matrix(axis, ncol = 2)
+    if (!add && (is.null(ylim) || all(is.na(ylim)))) { # compute and set ylim
+        ylim <- range(unlist(EvalInterval.function(fun,Xlim, vectorized)), na.rm = TRUE)
+        setView_ylim(ylim)
     }
 
-    if (is.null(mfrow)) {
-        nc <- round(sqrt(nrow(axis)))
-        nl <- ceiling(nrow(axis)/nc)
-        mfrow <- c(nc, nl)
+    if ( (length(levels)==1) && is.wholenumber(levels)) {
+        levels = pretty(range(unlist(EvalInterval.function(fun,Xlim,vectorized,D)),na.rm=TRUE), levels)
+        if (length(col_levels) != length(levels)-1) {
+            warning(paste0("col_levels reset to ",length(levels)-1," values"))
+            if (length(col_levels) == 2)
+                col_levels = cols.levels(col_levels[1],col_levels[2],levels,fill=TRUE)
+            else
+                col_levels = col.levels(col_levels[1],levels,fill=TRUE)
+        }
     }
 
-    if (length(col_levels) == length(levels)-1)
+    if (length(col_levels) == (length(levels)-1))
         col_fills = col_levels
     else if (length(col_levels) == 1)
-        col_fills = colorRampPalette(c(rgb(1,1,1,0),col_levels), alpha=TRUE)(levels) # from white transparent to col_level
+        col_fills = colorRampPalette(c(rgb(1,1,1,0),col_levels), alpha=TRUE)(length(levels)-1) # from white transparent to col_level
     else if (length(col_levels) == 2)
-        col_fills =  colorRampPalette(col_levels)(levels)
+        col_fills =  colorRampPalette(col_levels)(length(levels)-1)
     else
-        stop("col_levels must be a vector of length 1, 2 or levels.")
-
-    if (!isTRUE(add)) {
-        if (D>2) {
-            close.screen( all.screens = TRUE )
-            split.screen(figs = mfrow)
-        }
-        assign(".split.screen.lim",matrix(NaN,ncol=6,nrow=D),envir=DiceView.env) # xmin,xmax,ymin,ymax matrix of limits, each row for one dim combination
-    }
-
-    if (!exists(".split.screen.lim",envir=DiceView.env))
-        assign(".split.screen.lim",matrix(NaN,ncol=6,nrow=D),envir=DiceView.env)
+        stop("col_levels must be a vector of length 1, 2 or #levels-1 (was ",length(col_levels),", but #levels was ",length(levels),").")
 
     npoints <- rep(npoints, length.out = D)
-
-    ## find limits: 'rx' is matrix with min in row 1 and max in row 2
-    if(!is.null(Xlim))
-        rx <- matrix(Xlim,nrow=2,ncol=D)
-    else {
-        rx <- matrix(c(0,1),nrow=2,ncol=D)
-        if (!is.null(center)) # ensure center is included in Xlim
-            for (i in 1:D) {
-                rx[1,i] <- min(rx[1,i],center[i])
-                rx[2,i] <- max(rx[2,i],center[i])
-            }
-        }
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
-
-    zlim <- c(NA, NA) #Not used for this kind of plot
+    drx <- unlist(Xlim[2, ]) - unlist(Xlim[1, ])
 
     ## define X & y labels
     if (is.null(ylab)) ylab <- "y"
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
 
-    ## Added by YD (as in sectionview3d.km)
-    if (is.null(center)) {
-        center <- rep(0, D)
-        names(center) <- Xlab
-    }
-
     ## try to find a good formatted value 'fcenter' for 'center'
     fcenter <- tryFormat(x = center, drx = drx)
 
@@ -139,19 +104,11 @@ filledcontourview.function <- function(fun, vectorized=FALSE,
         ind.nonfix <- (1:D) %in% c(d[1], d[2])
         ind.nonfix <- !ind.nonfix
 
-        xlim <- unlist(rx[c("min","max"), d[1]])
-        if (isTRUE(add)) {
-            xlim['min'] <- min(xlim['min'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,1:2]))
-            xlim['max'] <- max(xlim['max'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,1:2]))
-        }
-        ylim <- unlist(rx[c("min","max"), d[2]])
-        if (isTRUE(add)) {
-            ylim['min'] <- min(ylim['min'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,3:4]))
-            ylim['max'] <- max(ylim['max'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,3:4]))
-        }
+        x1lim <- Xlim[, d[1]]
+        x2lim <- Xlim[, d[2]]
 
-        xd1 <- seq(from = as.numeric(xlim['min']), to = as.numeric(xlim['max']), length.out = npoints[1])
-        xd2 <- seq(from = as.numeric(ylim['min']), to = as.numeric(ylim['max']), length.out = npoints[2])
+        xd1 <- seq(from = as.numeric(x1lim[1]), to = as.numeric(x1lim[2]), length.out = npoints[d[1]])
+        xd2 <- seq(from = as.numeric(x2lim[1]), to = as.numeric(x2lim[2]), length.out = npoints[d[2]])
 
         x <- data.frame(t(matrix(as.numeric(center), nrow = D, ncol = npoints_all)))
         if (!is.null(center)) if(!is.null(names(center))) names(x) <- names(center)
@@ -168,7 +125,7 @@ filledcontourview.function <- function(fun, vectorized=FALSE,
         if (is.null(title)){
             title_d <- paste(collapse = "~",sep = "~", ylab, paste(collapse = ",", sep = ",", Xlab[d[1]], Xlab[d[2]]))
             if (D>2) {
-                title_d <-  paste(collapse = " | ", sep = " | ", title_d, paste(collapse=',',Xlab[ind.nonfix],'=', fcenter[ind.nonfix]))
+                title_d <-  paste(collapse =title_sep, sep =title_sep, title_d, paste(collapse=',',Xlab[ind.nonfix],'=', fcenter[ind.nonfix]))
             }
         } else {
             title_d <-  title
@@ -176,36 +133,31 @@ filledcontourview.function <- function(fun, vectorized=FALSE,
 
         ## plot mean
         if (isTRUE(add)) {
-            # re-use global settings for limits of this screen
-            .split.screen.lim = get(x=".split.screen.lim",envir=DiceView.env)
-            xlim <- c(.split.screen.lim[d,1],.split.screen.lim[d,2])
-            ylim <- c(.split.screen.lim[d,3],.split.screen.lim[d,4])
-            zlim <- c(.split.screen.lim[d,5],.split.screen.lim[d,6])
-            .filled.contour(x = xd1,y = xd2, z = if (!all(is.na(F_x$yd))) F_x$yd else F_x$yd_low,
-                            col = col_fills,
+            if (!all(is.na(F_x$yd))) {
+                contour(x = xd1,y = xd2, z = F_x$yd,
+                        xlim = x1lim, ylim = x2lim, zlim = ylim,
+                        col = col_fills,  lty = lty_levels,
+                        levels = levels,
+                        add=TRUE,
+                        ...)
+                .filled.contour(x = xd1,y = xd2, z = F_x$yd,
+                            col = if (is.na(add_fading)) col_fills else fades(col_fills,alpha=1-add_fading),
                             levels = levels)
-            if (lty_levels>0)
-                contour(x = xd1,y = xd2, z = if (!all(is.na(F_x$yd))) F_x$yd else F_x$yd_low,
-                    xlim = xlim, ylim = ylim, zlim = zlim,
-                    col = col_fills,  lty = lty_levels,
-                    levels = levels,
-                    add=TRUE,
-                    ...)
+            }
         } else {
-            xlim = rx[,d[1]]
-            ylim = rx[,d[2]]
-            eval(parse(text=paste(".split.screen.lim[",id,",] = matrix(c(",xlim[1],",",xlim[2],",",ylim[1],",",ylim[2],",",zlim[1],",",zlim[2],"),nrow=1)")),envir=DiceView.env)
-            contour(x = xd1, y = xd2, z = if (!all(is.na(F_x$yd))) F_x$yd else F_x$yd_low,
+            if (!all(is.na(F_x$yd))) {
+                contour(x = xd1, y = xd2, z =  F_x$yd,
                     xlab = Xlab[d[1]], ylab = Xlab[d[2]],
-                    xlim = xlim, ylim = ylim,
+                    xlim = x1lim, ylim = x2lim, zlim = ylim,
                     main = title_d,
                     col = col_fills,  lty = lty_levels,
                     levels = levels,
                     add=FALSE,
                     ...)
-            .filled.contour(x = xd1, y = xd2, z = if (!all(is.na(F_x$yd))) F_x$yd else F_x$yd_low,
+                .filled.contour(x = xd1, y = xd2, z = F_x$yd,
                     col = col_fills,
                     levels = levels)
+            }
             if(D>2) {
                 abline(v=center[d[1]],col=col_center,lty=lty_center)
                 abline(h=center[d[2]],col=col_center,lty=lty_center)
@@ -213,11 +165,24 @@ filledcontourview.function <- function(fun, vectorized=FALSE,
         }
 
 	    if (!all(is.na(F_x$yd_err))) {
-                col_fills_rgba = col2rgb("white")
-                col_err = rgb(col_fills_rgba[1]/255,col_fills_rgba[2]/255,col_fills_rgba[3]/255,seq(from=0,to=1,length=length(levels)))
-                image(x = xd1,y = xd2, z = F_x$yd_err,
-                      col = col_err, breaks=seq(from=min(F_x$yd_err),to=max(F_x$yd_err),length=length(col_err)+1),
-                      add=TRUE)
+                col_fills_rgba = col2rgb(col_interval)
+                fading = col_fading_interval #(add_fading + (1-add)*(1-add_fading))# * col_fading_interval
+                col_err = rgb(col_fills_rgba[1]/255,col_fills_rgba[2]/255,col_fills_rgba[3]/255,
+                              #seq(from=(1-fading)/2, to=1-(1-fading)/2,,length(levels)))
+                              seq(from=0, to=fading,,length(levels)-1))
+                filledcontourview.function(fun = function(x) {
+                        p = EvalInterval.function(fun, x, vectorized)
+                        p$y_up-p$y_low
+                    }, vectorized=vectorized,
+                    npoints = npoints,
+                    levels = seq(from=min(F_x$yd_err),to=max(F_x$yd_err),length=length(col_err)+1),
+                    col_levels = col_err, lty_levels=0,
+                    col_fading_interval=0,
+                    add=TRUE, add_fading = NA)
+
+                # image(x = xd1,y = xd2, z = F_x$yd_err,
+                #       col = col_err, breaks=seq(from=min(F_x$yd_err),to=max(F_x$yd_err),length=length(col_err)+1),
+                #       add=TRUE)
 	    }
     }
 }
@@ -250,23 +215,28 @@ filledcontourview.km <- function(km_model, type = "UK",
                            center = NULL,
                            axis = NULL,
                            npoints = 21,
-                           levels = pretty(km_model@y, 10),
+                           levels = pretty(c(km_model@y+2*sqrt(km_model@covariance@sd2), km_model@y-2*sqrt(km_model@covariance@sd2)), 10),
                            col_points = if (!is.null(col) & length(col)==1) col else "red",
-                           col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                           col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                            col = NULL,
                            conf_level = 0.5,
                            conf_fading = 0.5,
                            bg_fading = 1,
                            mfrow = NULL,
                            Xlab = NULL, ylab = NULL,
-                           Xlim = NULL,
-                           title = NULL,
+                           Xlim = NULL, ylim=NULL,
+                           title = NULL, title_sep = " | ",
                            add = FALSE,
                            ...) {
-    if (length(levels)==1) {
-        levels = pretty(km_model@y, levels)
-        if (length(col_levels) != length(levels)-1)
-            col_levels = col.levels(col_levels,levels)
+    if ( (length(levels)==1) && is.wholenumber(levels)) {
+        levels = pretty(c(km_model@y+2*sqrt(km_model@covariance@sd2), km_model@y-2*sqrt(km_model@covariance@sd2)), levels)
+        if (length(col_levels) != length(levels)-1) {
+            warning(paste0("col_levels reset to ",length(levels)-1," values"))
+            if (length(col_levels) == 2)
+                col_levels = cols.levels(col_levels[1],col_levels[2],levels,fill=TRUE)
+            else
+                col_levels = col.levels(col_levels[1],levels,fill=TRUE)
+        }
     }
 
     X_doe <- km_model@X
@@ -283,22 +253,22 @@ filledcontourview.km <- function(km_model, type = "UK",
         sdy_doe <- 0
     }
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
+
+    if (is.null(ylim) && !add) {
+        ymin <- min(y_doe-3*sdy_doe)
+        ymax <- max(y_doe+3*sdy_doe)
+        ylim <- c(ymin, ymax)
+    }
 
     ## define X & y labels
     if (is.null(ylab)) ylab <- names(y_doe)
     if (is.null(Xlab)) Xlab <- names(X_doe)
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
-
     if (is.null(conf_fading) ||
         length(conf_fading) != length(conf_level))
         conf_fading <- rep(0.5/length(conf_level), length(conf_level))
@@ -308,16 +278,10 @@ filledcontourview.km <- function(km_model, type = "UK",
         fun = function(x) {
             DiceKriging::predict.km(km_model,type=type,newdata=x,checkNames=FALSE)$mean
         }, vectorized=TRUE,
-    dim = D, center = center,axis = axis,npoints = npoints,
+    center = center, axis = axis, mfrow = mfrow, Xlim = Xlim, ylim=ylim,
     levels = levels, col_levels = col_levels,
-    mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-    Xlim = rx, title = title, add = add, ...)
-
-    contourview.matrix(X = X_doe, y = cbind(y_doe, sdy_doe),
-                       dim = D, center = center, axis = axis,
-                       col_points = col_points,
-                       bg_fading = bg_fading,
-                       mfrow = mfrow, Xlim = rx, add=TRUE)
+    Xlab = Xlab, ylab = ylab,
+    title = title, title_sep  = title_sep, add = add, ...)
 
     # plot confidence bands
     for (l in conf_level) {
@@ -325,11 +289,16 @@ filledcontourview.km <- function(km_model, type = "UK",
                 p = DiceKriging::predict.km(km_model,type=type,newdata=x,checkNames=FALSE)
                 cbind(p$mean-qnorm(1-(1-l)/2) * p$sd, p$mean+qnorm(1-(1-l)/2) * p$sd)
             }, vectorized=TRUE,
-            dim = D, center = center,axis = axis,npoints = npoints,
+            npoints = npoints,
             levels = levels, col_levels = col_levels,
             col_fading_interval=conf_fading,
-            mfrow = mfrow, Xlim = rx, add = TRUE)
+            add = TRUE)
     }
+
+    contourview.matrix(X = X_doe, y = cbind(y_doe, sdy_doe),
+                       col_points = col_points,
+                       bg_fading = bg_fading,
+                       add = TRUE)
 }
 
 #' @param libKriging_model an object of class \code{"Kriging"}, \code{"NuggetKriging"} or \code{"NoiseKriging"}.
@@ -341,23 +310,29 @@ filledcontourview_libKriging <- function(libKriging_model,
                            center = NULL,
                            axis = NULL,
                            npoints = 21,
-                           levels = pretty( libKriging_model$y() , 10),
+                           levels = pretty( c(libKriging_model$y() + 2*sqrt(libKriging_model$sigma2()), libKriging_model$y() - 2*sqrt(libKriging_model$sigma2())), 10),
                            col_points = if (!is.null(col) & length(col)==1) col else "red",
-                           col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                           col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                            col = NULL,
                            conf_level = 0.5,
                            conf_fading = 0.5,
                            bg_fading = 1,
                            mfrow = NULL,
                            Xlab = NULL, ylab = NULL,
-                           Xlim = NULL,
-                           title = NULL,
+                           Xlim = NULL, ylim=NULL,
+                           title = NULL, title_sep = " | ",
                            add = FALSE,
                            ...) {
-    if (length(levels)==1) {
-        levels = pretty(libKriging_model$y(), levels)
-        if (length(col_levels) != length(levels)-1)
-            col_levels = col.levels(col_levels,levels)
+    if ( (length(levels)==1) && is.wholenumber(levels)) {
+        levels = pretty(c(libKriging_model$y()-2*sqrt(libKriging_model$sigma2()),
+                          libKriging_model$y()+2*sqrt(libKriging_model$sigma2())), levels)
+        if (length(col_levels) != length(levels)-1) {
+            warning(paste0("col_levels reset to ",length(levels)-1," values"))
+            if (length(col_levels) == 2)
+                col_levels = cols.levels(col_levels[1],col_levels[2],levels,fill=TRUE)
+            else
+                col_levels = col.levels(col_levels[1],levels,fill=TRUE)
+        }
     }
 
     X_doe <- libKriging_model$X()
@@ -375,11 +350,21 @@ filledcontourview_libKriging <- function(libKriging_model,
     } else
         stop(paste0("Kriging model of class ",class(libKriging_model)," is not yet supported."))
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
+
+    if (is.null(ylim) && !add) {
+        if (is.null(sdy_doe))
+            ylim = range(y_doe)
+        else {
+            ymin <- min(y_doe-3*sdy_doe)
+            ymax <- max(y_doe+3*sdy_doe)
+            ylim <- c(ymin, ymax)
+        }
+    }
 
     ## define X & y labels
     if (is.null(ylab)) ylab <- names(y_doe)
@@ -387,12 +372,6 @@ filledcontourview_libKriging <- function(libKriging_model,
     if (is.null(Xlab)) Xlab <- names(X_doe)
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
-
     if (is.null(conf_fading) ||
         length(conf_fading) != length(conf_level))
         conf_fading <- rep(0.5/length(conf_level), length(conf_level))
@@ -400,17 +379,11 @@ filledcontourview_libKriging <- function(libKriging_model,
     filledcontourview.function(fun = function(x) {
             rlibkriging::predict(libKriging_model,x,return_stdev=FALSE)$mean
         }, vectorized=TRUE,
-        dim = D, center = center,axis = axis,npoints = npoints,
+        center = center, axis = axis, mfrow = mfrow, Xlim = Xlim, ylim=ylim,
+        npoints = npoints,
         levels = levels, col_levels = col_levels,
-        mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-        Xlim = rx, title = title, add = add, ...)
-
-    # plot design points
-    contourview.matrix(X = X_doe, y = y_doe,
-                       dim = D, center = center, axis = axis,
-                       col_points = col_points,
-                       bg_fading = bg_fading,
-                       mfrow = mfrow, Xlim = rx, add=TRUE)
+        Xlab = Xlab, ylab = ylab,
+        title = title, title_sep  = title_sep, add = add, ...)
 
     # plot confidence bands
     for (l in conf_level) {
@@ -418,11 +391,17 @@ filledcontourview_libKriging <- function(libKriging_model,
                 p = rlibkriging::predict(libKriging_model,x,return_stdev=TRUE)
                 cbind(p$mean-qnorm(1-(1-l)/2) * p$stdev, p$mean+qnorm(1-(1-l)/2) * p$stdev)
             }, vectorized=TRUE,
-            dim = D, center = center,axis = axis,npoints = npoints,
+            npoints = npoints,
             levels = levels, col_levels = col_levels,
             col_fading_interval=conf_fading,
-            mfrow = mfrow, Xlim = rx, add = TRUE)
+            add = TRUE)
     }
+
+    # plot design points
+    contourview.matrix(X = X_doe, y = y_doe,
+                       col_points = col_points,
+                       bg_fading = bg_fading,
+                       add = TRUE)
 }
 
 #' @param Kriging_model an object of class \code{"Kriging"}.
@@ -454,15 +433,15 @@ filledcontourview.Kriging <- function(Kriging_model,
                                    npoints = 21,
                                    levels = pretty( Kriging_model$y() , 10),
                                    col_points = if (!is.null(col) & length(col)==1) col else "red",
-                                   col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                                   col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                                    col = NULL,
                                    conf_level = 0.5,
                                    conf_fading = 0.5,
                                    bg_fading = 1,
                                    mfrow = NULL,
                                    Xlab = NULL, ylab = NULL,
-                                   Xlim = NULL,
-                                   title = NULL,
+                                   Xlim = NULL, ylim=NULL,
+                                   title = NULL, title_sep = " | ",
                                    add = FALSE,
                                    ...) {
     filledcontourview_libKriging(libKriging_model = Kriging_model,
@@ -478,7 +457,7 @@ filledcontourview.Kriging <- function(Kriging_model,
                            bg_fading = bg_fading,
                            mfrow = mfrow,
                            Xlab = Xlab, ylab = ylab,
-                           Xlim = Xlim,
+                           Xlim = Xlim, ylim=ylim,
                            title = title,
                            add = add,
                            ...)
@@ -513,15 +492,15 @@ filledcontourview.NuggetKriging <- function(NuggetKriging_model,
                                 npoints = 21,
                                 levels = pretty( NuggetKriging_model$y() , 10),
                                 col_points = if (!is.null(col) & length(col)==1) col else "red",
-                                col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                                col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                                 col = NULL,
                                 conf_level = 0.5,
                                 conf_fading = 0.5,
                                 bg_fading = 1,
                                 mfrow = NULL,
                                 Xlab = NULL, ylab = NULL,
-                                Xlim = NULL,
-                                title = NULL,
+                                Xlim = NULL, ylim=NULL,
+                                title = NULL, title_sep = " | ",
                                 add = FALSE,
                                 ...) {
     filledcontourview_libKriging(libKriging_model = NuggetKriging_model,
@@ -537,7 +516,7 @@ filledcontourview.NuggetKriging <- function(NuggetKriging_model,
                            bg_fading = bg_fading,
                            mfrow = mfrow,
                            Xlab = Xlab, ylab = ylab,
-                           Xlim = Xlim,
+                           Xlim = Xlim, ylim=ylim,
                            title = title,
                            add = add,
                            ...)
@@ -572,15 +551,15 @@ filledcontourview.NoiseKriging <- function(NoiseKriging_model,
                                       npoints = 21,
                                       levels = pretty( NoiseKriging_model$y() , 10),
                                       col_points = if (!is.null(col) & length(col)==1) col else "red",
-                                      col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                                      col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                                       col = NULL,
                                       conf_level = 0.5,
                                       conf_fading = 0.5,
                                       bg_fading = 1,
                                       mfrow = NULL,
                                       Xlab = NULL, ylab = NULL,
-                                      Xlim = NULL,
-                                      title = NULL,
+                                      Xlim = NULL, ylim=NULL,
+                                      title = NULL, title_sep = " | ",
                                       add = FALSE,
                                       ...) {
     filledcontourview_libKriging(libKriging_model = NoiseKriging_model,
@@ -596,7 +575,7 @@ filledcontourview.NoiseKriging <- function(NoiseKriging_model,
                            bg_fading = bg_fading,
                            mfrow = mfrow,
                            Xlab = Xlab, ylab = ylab,
-                           Xlim = Xlim,
+                           Xlim = Xlim, ylim=ylim,
                            title = title,
                            add = add,
                            ...)
@@ -628,21 +607,26 @@ filledcontourview.glm <- function(glm_model,
                            npoints = 21,
                            levels = pretty( glm_model$fitted.values , 10),
                            col_points = if (!is.null(col) & length(col)==1) col else "red",
-                           col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                           col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                            col = NULL,
                            conf_level = 0.5,
                            conf_fading = 0.5,
                            bg_fading = 1,
                            mfrow = NULL,
                            Xlab = NULL, ylab = NULL,
-                           Xlim = NULL,
-                           title = NULL,
+                           Xlim = NULL, ylim=NULL,
+                           title = NULL, title_sep = " | ",
                            add = FALSE,
                            ...) {
-    if (length(levels)==1) {
+    if ( (length(levels)==1) && is.wholenumber(levels)) {
         levels = pretty(glm_model$fitted.values, levels)
-        if (length(col_levels) != length(levels)-1)
-            col_levels = col.levels(col_levels,levels)
+        if (length(col_levels) != length(levels)-1) {
+            warning(paste0("col_levels reset to ",length(levels)-1," values"))
+            if (length(col_levels) == 2)
+                col_levels = cols.levels(col_levels[1],col_levels[2],levels,fill=TRUE)
+            else
+                col_levels = col.levels(col_levels[1],levels,fill=TRUE)
+        }
     }
 
     # Get X & y labels
@@ -663,17 +647,11 @@ filledcontourview.glm <- function(glm_model,
     y_doe <- do.call(cbind,lapply(ylab,function(yn)glm_model$data[[yn]]))
     colnames(y_doe) <- ylab
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
-
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
 
     if (is.null(conf_fading) ||
         length(conf_fading) != length(conf_level))
@@ -686,20 +664,14 @@ filledcontourview.glm <- function(glm_model,
             colnames(x) <- Xlab
             predict.glm(glm_model, newdata=x, se.fit=FALSE)
         }, vectorized=TRUE,
-        dim = D, center = center,axis = axis, npoints = npoints,
+        center = center,axis = axis, mfrow = mfrow, Xlim = Xlim, ylim=range(y_doe),
+        npoints = npoints,
         levels = levels, col_levels = col_levels,
-        mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-        Xlim = rx, title = title, add = add, ...)
+        Xlab = Xlab, ylab = ylab,
+        title = title, title_sep  = title_sep, add = add, ...)
 
-    # plot design points
-    contourview.matrix(X = X_doe, y = y_doe,
-                       dim = D, center = center, axis = axis,
-                       col_points = col_points,
-                       bg_fading = bg_fading,
-                       mfrow = mfrow,
-                       Xlim = rx,
-                       add=TRUE)
-                           # plot confidence bands
+
+    # plot confidence bands
     for (l in conf_level) {
         filledcontourview.function(fun = function(x) {
                 x = as.data.frame(x)
@@ -707,11 +679,17 @@ filledcontourview.glm <- function(glm_model,
                 p = predict.glm(glm_model, newdata=x, se.fit=TRUE)
                 cbind(p$fit-qnorm(1-(1-l)/2) * p$se.fit, p$fit+qnorm(1-(1-l)/2) * p$se.fit)
             }, vectorized=TRUE,
-            dim = D, center = center,axis = axis,npoints = npoints,
+            npoints = npoints,
             levels = levels, col_levels = col_levels,
             col_fading_interval=conf_fading,
-            mfrow = mfrow, Xlim = rx, add = TRUE)
+            add = TRUE)
     }
+
+    # plot design points
+    contourview.matrix(X = X_doe, y = y_doe,
+                       col_points = col_points,
+                       bg_fading = bg_fading,
+                       add = TRUE)
 }
 
 
@@ -742,19 +720,24 @@ filledcontourview.list <- function(modelFit_model,
                             npoints = 21,
                             levels = pretty( modelFit_model$data$Y , 10),
                             col_points = if (!is.null(col) & length(col)==1) col else "red",
-                            col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels) else col.levels("blue",levels),
+                            col_levels = if (!is.null(col) & length(col)==1) col.levels(col,levels, fill=TRUE) else if (!is.null(col) & length(col)==2) cols.levels(col[1],col[2],levels,fill=TRUE) else col.levels("blue",levels, fill=TRUE),
                             col = NULL,
                             bg_fading = 1,
                             mfrow = NULL,
                             Xlab = NULL, ylab = NULL,
-                            Xlim = NULL,
-                            title = NULL,
+                            Xlim = NULL, ylim=NULL,
+                            title = NULL, title_sep = " | ",
                             add = FALSE,
                             ...) {
-    if (length(levels)==1) {
+    if ( (length(levels)==1) && is.wholenumber(levels)) {
         levels = pretty(modelFit_model$data$Y, levels)
-        if (length(col_levels) != length(levels)-1)
-            col_levels = col.levels(col_levels,levels)
+        if (length(col_levels) != length(levels)-1) {
+            warning(paste0("col_levels reset to ",length(levels)-1," values"))
+            if (length(col_levels) == 2)
+                col_levels = cols.levels(col_levels[1],col_levels[2],levels,fill=TRUE)
+            else
+                col_levels = col.levels(col_levels[1],levels,fill=TRUE)
+        }
     }
 
     X_doe <- modelFit_model$data$X
@@ -769,17 +752,11 @@ filledcontourview.list <- function(modelFit_model,
     if (is.null(Xlab)) Xlab <- names(X_doe)
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
-
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
 
     filledcontourview.function(
         fun = function(x) {
@@ -787,17 +764,17 @@ filledcontourview.list <- function(modelFit_model,
             colnames(x) <- Xlab
             DiceEval::modelPredict(modelFit_model, x)
         }, vectorized=TRUE,
-        dim = D, center = center,axis = axis, npoints = npoints,
+        center = center,axis = axis,mfrow = mfrow,Xlim = Xlim, ylim=range(y_doe),
+        npoints = npoints,
         levels = levels, col_levels = col_levels,
-        mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-        Xlim = rx, title = title, add = add, ...)
+        Xlab = Xlab, ylab = ylab,
+        title = title, title_sep  = title_sep, add = add, ...)
 
     # plot design points
     contourview.matrix(X = X_doe, y = y_doe,
-                       dim = D, center = center, axis = axis,
                        col_points = col_points,
                        bg_fading = bg_fading,
-                       mfrow = mfrow, Xlim = rx, add=TRUE)
+                       add = TRUE)
 }
 
 
@@ -819,7 +796,7 @@ if(!isGeneric("filledcontourview")) {
 #' @export
 #' @examples
 #' ## A 2D example - Branin-Hoo function
-#' filledcontourview(branin, dim=2, levels=30, col='black')
+#' filledcontourview(branin, levels=30, col='black')
 #'
 #' \dontrun{
 #' ## a 16-points factorial design, and the corresponding response
@@ -832,26 +809,26 @@ if(!isGeneric("filledcontourview")) {
 #' ## model: km
 #' model <- DiceKriging::km(design = design.fact, response = y)
 #' filledcontourview(model, levels=30)
-#' filledcontourview(branin, dim=2, levels=30, col='red', add=TRUE)
+#' filledcontourview(branin, levels=30, col='red', add=TRUE)
 #' }
 #'
 #' if (requireNamespace("rlibkriging")) { library(rlibkriging)
 #' ## model: Kriging
 #' model <- Kriging(X = as.matrix(design.fact), y = as.matrix(y), kernel="matern3_2")
 #' filledcontourview(model, levels=30)
-#' filledcontourview(branin, dim=2, levels=30, col='red', add=TRUE)
+#' filledcontourview(branin, levels=30, col='red', add=TRUE)
 #' }
 #'
 #' ## model: glm
 #' model <- glm(y ~ 1+ x1 + x2 + I(x1^2) + I(x2^2) + x1*x2, data=cbind(y,design.fact))
 #' filledcontourview(model, levels=30)
-#' filledcontourview(branin, dim=2, levels=30, col='red', add=TRUE)
+#' filledcontourview(branin, levels=30, col='red', add=TRUE)
 #'
 #' if (requireNamespace("DiceEval")) { library(DiceEval)
 #' ## model: StepLinear
 #' model <- modelFit(design.fact, y, type = "StepLinear")
 #' filledcontourview(model, levels=30)
-#' filledcontourview(branin, dim=2, levels=30, col='red', add=TRUE)
+#' filledcontourview(branin, levels=30, col='red', add=TRUE)
 #' }
 #' }
 #'
