@@ -1,6 +1,5 @@
 #' @param fun a function or 'predict()'-like function that returns a simple numeric, or an interval, or mean and standard error: list(mean=...,se=...).
 #' @param vectorized is fun vectorized?
-#' @param dim input variables dimension of the model or function.
 #' @param col_fun color of the function plot.
 #' @param col_fading_interval an optional factor of alpha (color channel) fading used to plot function output intervals (if any).
 #' @template sectionview3d-doc
@@ -16,103 +15,61 @@
 #' y <- x1 + x2 + rnorm(15)
 #' model <- lm(y ~ x1 + x2)
 #'
-#' DiceView:::open3d();
-#' DiceView:::plot3d(x1,x2,y)
 #' sectionview3d(function(x) sum(x),
-#'                      dim=2, Xlim=cbind(range(x1),range(x2)), add=TRUE, col='black')
+#'                        Xlim=cbind(range(x1),range(x2)), col='black')
+#' DiceView:::.plot3d(x1, x2, y)
 #'
 #' sectionview3d(function(x) {
 #'                       x = as.data.frame(x)
 #'                       colnames(x) <- all.vars(model$call)[-1]
 #'                       p = predict.lm(model, newdata=x, se.fit=TRUE)
 #'                       list(mean=p$fit, se=p$se.fit)
-#'                     }, vectorized=TRUE, dim=2,
-#'               Xlim=cbind(range(x1),range(x2)), add=TRUE)
+#'                     }, vectorized=TRUE,
+#'               add=TRUE)
 #'
 sectionview3d.function <- function(fun, vectorized=FALSE,
-                                dim = NULL,
                              center = NULL,
                              axis = NULL,
                              npoints = 21,
                              col_fun = if (!is.null(col)) col else "blue",
                              col = NULL,
                              col_fading_interval = 0.5,
-                             mfrow = NULL,
+                             mfrow = c(1,1),
                              Xlab = NULL, ylab = NULL,
-                             Xlim = NULL, ylim = NULL,
-                             title = NULL,
+                             Xlim = if (!add) matrix(c(0,1),2,2) else NULL, ylim = NULL,
+                             title = NULL, title_sep = " | ",
                              add = FALSE,
                              engine3d = NULL,
                              ...) {
-    engine3d = load3d(engine3d)
+    engine3d = .load3d(engine3d)
 
-    if (!is.null(dim)) {
-        D <- dim
-        if (is.null(center))
-            if (D != 2) stop("Section center in 'section' required for >2-D model.")
-    } else {
-        if (is.null(center))
-            stop("dim or center must be specified.")
-        else
-            D <- length(center)
-    }
+    # setup for center/Xlim/axis/mfrow
+    view = getView(2, add, center, Xlim, ylim, axis, mfrow)
+    D = view$D
+    center = view$center
+    Xlim = view$Xlim
+    ylim = view$ylim
+    axis = view$axis
+    mfrow = view$mfrow
 
     if (D == 1) stop("for a model with dim 1, use 'sectionview'")
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        ## added by YD for the vector case
-        axis <- matrix(axis, ncol = 2)
+    if (engine3d == "rgl" && D>2 && nrow(axis)!=1) {
+        stop("only one plot supported for rgl engine. Set axis=c(.,.)")
     }
 
-    if (engine3d != "rgl") {
-        if (is.null(mfrow)) {
-            nc <- round(sqrt(nrow(axis)))
-            nl <- ceiling(nrow(axis)/nc)
-            mfrow <- c(nc, nl)
-        }
+    if (!add && (is.null(ylim) || all(is.na(ylim)))) { # compute and set ylim
+        ylim <- range(unlist(EvalInterval.function(fun,Xlim, vectorized)), na.rm = TRUE)
+        setView_ylim(ylim)
     }
 
-    if (!isTRUE(add)) {
-        if(engine3d != "rgl"){
-            close.screen( all.screens = TRUE )
-            split.screen(figs = mfrow)
-        }
-        assign(".split.screen.lim",matrix(NaN,ncol=6,nrow=D),envir=DiceView.env) # xmin,xmax,ymin,ymax matrix of limits, each row for one dim combination
-    }
-
-    if (!exists(".split.screen.lim",envir=DiceView.env))
-        assign(".split.screen.lim",matrix(NaN,ncol=6,nrow=D),envir=DiceView.env)
-
     npoints <- rep(npoints, length.out = D)
-
-    ## find limits: 'rx' is matrix with min in row 1 and max in row 2
-    if(!is.null(Xlim))
-        rx <- matrix(Xlim,nrow=2,ncol=D)
-    else {
-        rx <- matrix(c(0,1),nrow=2,ncol=D)
-        if (!is.null(center)) # ensure center is included in Xlim
-            for (i in 1:D) {
-                rx[1,i] <- min(rx[1,i],center[i])
-                rx[2,i] <- max(rx[2,i],center[i])
-            }
-    }
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
-
-    zlim <- range(unlist(EvalInterval.function(fun,rbind(rx,center), vectorized)), na.rm = TRUE)
+    drx <- unlist(Xlim[2, ]) - unlist(Xlim[1, ])
 
     ## define X & y labels
     if (is.null(ylab)) ylab <- "y"
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
 
-    ## Added by YD (as in sectionview3d.km)
-    if (is.null(center)) {
-        center <- rep(0, D)
-        names(center) <- Xlab
-    }
-
     ## try to find a good formatted value 'fcenter' for 'center'
     fcenter <- tryFormat(x = center, drx = drx)
 
@@ -128,19 +85,11 @@ sectionview3d.function <- function(fun, vectorized=FALSE,
         ind.nonfix <- (1:D) %in% c(d[1], d[2])
         ind.nonfix <- !ind.nonfix
 
-        xlim <- unlist(rx[c("min","max"), d[1]])
-        if (isTRUE(add)) {
-            xlim['min'] <- min(xlim['min'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,1:2]))
-            xlim['max'] <- max(xlim['max'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,1:2]))
-        }
-        ylim <- unlist(rx[c("min","max"), d[2]])
-        if (isTRUE(add)) {
-            ylim['min'] <- min(ylim['min'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,3:4]))
-            ylim['max'] <- max(ylim['max'],c(get(x=".split.screen.lim",envir=DiceView.env)[1,3:4]))
-        }
+        x1lim <- Xlim[, d[1]]
+        x2lim <- Xlim[, d[2]]
 
-        xd1 <- seq(from = as.numeric(xlim['min']), to = as.numeric(xlim['max']), length.out = npoints[1])
-        xd2 <- seq(from = as.numeric(ylim['min']), to = as.numeric(ylim['max']), length.out = npoints[2])
+        xd1 <- seq(from = as.numeric(x1lim[1]), to = as.numeric(x1lim[2]), length.out = npoints[d[1]])
+        xd2 <- seq(from = as.numeric(x2lim[1]), to = as.numeric(x2lim[2]), length.out = npoints[d[2]])
 
         x <- data.frame(t(matrix(as.numeric(center), nrow = D, ncol = npoints_all)))
         if (!is.null(center)) if(!is.null(names(center))) names(x) <- names(center)
@@ -154,45 +103,41 @@ sectionview3d.function <- function(fun, vectorized=FALSE,
         if (is.null(title)){
             title_d <- paste(collapse = "~",sep = "~", ylab, paste(collapse = ",", sep = ",", Xlab[d[1]], Xlab[d[2]]))
             if (D>2) {
-                title_d <-  paste(collapse = " | ", sep = " | ", title_d, paste(collapse=',',Xlab[ind.nonfix],'=', fcenter[ind.nonfix]))
+                title_d <-  paste(collapse =title_sep, sep =title_sep, title_d, paste(collapse=',',Xlab[ind.nonfix],'=', fcenter[ind.nonfix]))
             }
         } else {
             title_d <-  title
         }
 
-        ## plot mean
         if (isTRUE(add)) {
-            # re-use global settings for limits of this screen
-            .split.screen.lim = get(x=".split.screen.lim",envir=DiceView.env)
-            xlim <- c(.split.screen.lim[d,1],.split.screen.lim[d,2])
-            ylim <- c(.split.screen.lim[d,3],.split.screen.lim[d,4])
-            zlim <- c(.split.screen.lim[d,5],.split.screen.lim[d,6])
+            # do nothing
         } else {
-            eval(parse(text=paste(".split.screen.lim[",d,",] = matrix(c(",xlim[1],",",xlim[2],",",ylim[1],",",ylim[2],",",zlim[1],",",zlim[2],"),nrow=1)")),envir=DiceView.env)
-            open3d()
-            plot3d(x = x[ , 1], y = x[ , 2], z = if (!all(is.na(F_x$y))) F_x$y else F_x$y_low,
+            # empty plot
+            .open3d()
+            .plot3d(x = x[ , 1], y = x[ , 2], z = if (!all(is.na(F_x$y))) F_x$y else F_x$y_low,
                 xlab = Xlab[d[1]], ylab = Xlab[d[2]], zlab = ylab,
-                xlim = xlim, ylim = ylim, zlim = zlim,
+                xlim = x1lim, ylim = x2lim, zlim = ylim,
                 type = "n",
                 main = title_d,
                 col = col_fun,
                 ...)
         }
+        ## plot mean
 	    if (!any(is.na(F_x$yd)))
-            surface3d(x = xd1,y = xd2, z = F_x$yd,
+	        .surface3d(x = xd1,y = xd2, z = F_x$yd,
                     col = col_fun, alpha = 0.5,
                     box = FALSE)
 
-            ## 'confidence band' filled with the suitable color
+        ## 'confidence band' filled with the suitable color
 	    if (!all(is.na(F_x$yd_low)) && !all(is.na(F_x$yd_up))) {
-                surface3d(x = xd1,
+	        .surface3d(x = xd1,
                         y = xd2,
                         z =  F_x$yd_low,
                         col = col_fun,
                         alpha = col_fading_interval,
                         box = FALSE)
 
-                surface3d(x = xd1,
+	        .surface3d(x = xd1,
                         y = xd2,
                         z = F_x$yd_up,
                         col = col_fun,
@@ -227,16 +172,32 @@ sectionview3d.matrix <- function(X, y,
                              col = NULL,
                              col_fading_interval = 0.5,
                              bg_fading = 1,
-                             mfrow = NULL,
+                             mfrow = c(1,1),
                              Xlab = NULL, ylab = NULL,
-                             Xlim = NULL, ylim = NULL,
-                             title = NULL,
+                             Xlim = if (!add) matrix(c(0,1),2,2) else NULL, ylim = NULL,
+                             title = NULL, title_sep = " | ",
                              add = FALSE,
                              engine3d = NULL,
                              ...) {
-    engine3d = load3d(engine3d)
+    engine3d = .load3d(engine3d)
 
-    D <- ncol(X)
+    # setup for center/Xlim/axis/mfrow
+    view = getView(1, add, center, if (is.null(Xlim) && !add) apply(X, 2, range) else Xlim, if (is.null(ylim) && !add) ylim <- range(y) else ylim, axis, mfrow)
+    D = view$D
+    center = view$center
+    Xlim = view$Xlim
+    ylim = view$ylim
+    axis = view$axis
+    mfrow = view$mfrow
+
+    if (D == 1) stop("for a model with dim 1, use 'sectionview'")
+
+    if (engine3d == "rgl" && D>2 && nrow(axis)!=1) {
+        stop("only one plot supported for rgl engine. Set axis=c(.,.)")
+    }
+
+    if (ncol(X) != D)
+        stop(paste0("X must have ",D," columns."))
     n <- nrow(X)
 
     if (is.matrix(y) && ncol(y) == 2) {
@@ -248,41 +209,12 @@ sectionview3d.matrix <- function(X, y,
         y_up <- NA
     }
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
+    drx <- unlist(Xlim[2, ]) - unlist(Xlim[1, ])
 
     ## define X & y labels
     if (is.null(ylab) && !is.null(names(y))) ylab <- names(y)[1]
     if (is.null(Xlab)) Xlab <- names(X)
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
-
-    if (engine3d != "rgl") {
-        if (is.null(mfrow)) {
-            nc <- round(sqrt(nrow(axis)))
-            nl <- ceiling(nrow(axis)/nc)
-            mfrow <- c(nc, nl)
-        }
-    }
-
-    if (!isTRUE(add)) {
-        if(engine3d != "rgl"){
-            close.screen( all.screens = TRUE )
-            split.screen(figs = mfrow)
-        }
-        assign(".split.screen.lim",matrix(NaN,ncol=6,nrow=D),envir=DiceView.env) # xmin,xmax,ymin,ymax matrix of limits, each row for one dim combination
-    }
-
-    if (!exists(".split.screen.lim",envir=DiceView.env))
-        assign(".split.screen.lim",matrix(NaN,ncol=6,nrow=D),envir=DiceView.env)
-
     ## define X & y labels
     if (is.null(ylab)) ylab <- "y"
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
@@ -299,15 +231,13 @@ sectionview3d.matrix <- function(X, y,
         ind.nonfix <- (1:D) %in% c(d[1], d[2])
         ind.nonfix <- !ind.nonfix
 
-        xdmin <- unlist(rx["min", d])
-        xdmax <- unlist(rx["max", d])
-        xlim <- unlist(rx[ , d[1]])
-        ylim <- unlist(rx[ , d[2]])
+        x1lim = Xlim[, d[1]]
+        x2lim = Xlim[, d[2]]
 
         if (is.null(title)){
-            title_d <- paste(collapse = "~",sep = "~", ylab, Xlab[d])
-            if (D>1) {
-                title_d <-  paste(collapse = " | ", sep = " | ", title_d, paste(Xlab[-d], '=', fcenter[-d]))
+            title_d <- paste(collapse = "~",sep = "~", ylab, paste(collapse = ",", sep = ",", Xlab[d[1]], Xlab[d[2]]))
+            if (D>2) {
+                title_d <-  paste(collapse =title_sep, sep =title_sep, title_d, paste(collapse=',',Xlab[ind.nonfix],'=', fcenter[ind.nonfix]))
             }
         } else {
             title_d <- title
@@ -331,31 +261,25 @@ sectionview3d.matrix <- function(X, y,
         }
 
         if (isTRUE(add)) {
-            .split.screen.lim = get(x=".split.screen.lim",envir=DiceView.env)
-            xlim <- c(.split.screen.lim[d,1],.split.screen.lim[d,2])
-            ylim <- c(.split.screen.lim[d,3],.split.screen.lim[d,4])
-            zlim <- c(.split.screen.lim[d,5],.split.screen.lim[d,6])
-            points3d(x = X[ , d[1]], y = X[ , d[2]], z = if (!all(is.na(y))) y else y_low,
+            .points3d(x = X[ , d[1]], y = X[ , d[2]], z = if (!all(is.na(y))) y else y_low,
                      col = col_points,
                      alpha = alpha,
                      pch = 20, size=if (!all(is.na(y))) 3 else 0, box = FALSE,
-                     xlim=xlim, ylim=ylim, zlim=zlim)
+                     xlim=x1lim, ylim=x2lim, zlim=ylim)
         } else {
-            zlim = range(y)
-            eval(parse(text=paste(".split.screen.lim[",d,",] = matrix(c(",xlim[1],",",xlim[2],",",ylim[1],",",ylim[2],",",zlim[1],",",zlim[2],"),nrow=1)")),envir=DiceView.env)
-            open3d()
-            plot3d(x = X[ , d[1]], y = X[ , d[2]], z = if (!all(is.na(y))) y else y_low,
+            .open3d()
+            .plot3d(x = X[ , d[1]], y = X[ , d[2]], z = if (!all(is.na(y))) y else y_low,
                      col = col_points,
                      alpha = alpha,
                      pch = 20, size=if (!all(is.na(y))) 3 else 0, box = TRUE,
                      xlab=Xlab[d], ylab=ylab,
-                     xlim=xlim, ylim=ylim, zlim=zlim)
+                     xlim=x1lim, ylim=x2lim, zlim=ylim)
         }
 
         if (!all(is.na(y_low)) && !all(is.na(y_up)))
             #for (p in 1:length(conf_level)) {
                 for (i in 1:n) {
-                    lines3d(x = c(X[i, d[1]], X[i, d[1]]),
+                    .lines3d(x = c(X[i, d[1]], X[i, d[1]]),
                     y = c(X[i, d[2]], X[i, d[2]]),
                     z = c(y_low[i], y_up[i]),
                     col = col_points,
@@ -366,7 +290,7 @@ sectionview3d.matrix <- function(X, y,
         # else
         #     for (p in 1:length(conf_level)) {
         #         for (i in 1:n) {
-        #             points3d(x = X_doe[i, d[1]],
+        #             .points3d(x = X_doe[i, d[1]],
         #              y = X_doe[i, d[2]],
         #              z = y_doe[i],
         #              col = col_points,
@@ -397,28 +321,20 @@ sectionview3d.matrix <- function(X, y,
 #' sectionview3d("abline(h=0.25,col='red')")
 sectionview3d.character <- function(eval_str,
                                   axis = NULL,
-                                  mfrow = NULL,
+                                  mfrow = c(1,1),
                                   ...) {
 
     if (!exists(".split.screen.lim",envir=DiceView.env))
         stop(paste0("Cannot eval '",eval_str,"' when no previous sectionview() was called."))
-    else
-        .split.screen.lim = get(x=".split.screen.lim",envir=DiceView.env)
 
-    D <- nrow(.split.screen.lim)
-
-    if (is.null(axis)) {
-        axis <- matrix(1:D, ncol = 1)
-    } else {
-        ## added by YD for the vector case
-        axis <- matrix(axis, ncol = 1)
-    }
-
-    if (is.null(mfrow)) {
-        nc <- round(sqrt(D))
-        nl <- ceiling(D/nc)
-        mfrow <- c(nc, nl)
-    }
+    # setup for center/Xlim/axis/mfrow
+    view = getView(2, TRUE, NA, NA, NA, axis, mfrow)
+    D = view$D
+    center = view$center
+    Xlim = view$Xlim
+    ylim = view$ylim
+    axis = view$axis
+    mfrow = view$mfrow
 
     ## Each 'id' will produce a plot
     for (id in 1:dim(axis)[1]) {
@@ -427,22 +343,16 @@ sectionview3d.character <- function(eval_str,
 
         e = parent.frame()
         assign("d",d,envir=e)
-        assign("xlim",c(.split.screen.lim[d,1],.split.screen.lim[d,2]),envir=e)
-        assign("ylim",c(.split.screen.lim[d,3],.split.screen.lim[d,4]),envir=e)
-        assign("zlim",c(.split.screen.lim[d,5],.split.screen.lim[d,6]),envir=e)
+        assign("xlim",Xlim[,d[1]],envir=e)
+        assign("ylim",Xlim[,d[2]],envir=e)
+        assign("zlim",ylim,envir=e)
 
-        if (D>1) {
+        if (D>2) {
             screen(id, new=FALSE)
-
-            .split.screen.lim = get(x=".split.screen.lim",envir=DiceView.env)
-            xlim <- c(.split.screen.lim[d,1],.split.screen.lim[d,2])
-            ylim <- c(.split.screen.lim[d,3],.split.screen.lim[d,4])
-            zlim <- c(.split.screen.lim[d,5],.split.screen.lim[d,6])
-
-            plot3d(x=c(.split.screen.lim[d,1],.split.screen.lim[d,2]), y=c(.split.screen.lim[d,3],.split.screen.lim[d,4]), z=c(.split.screen.lim[d,5],.split.screen.lim[d,6]),
+            .plot3d(x=Xlim[,d[1]], y=Xlim[,d[2]],z=ylim,
                  type='n', box = FALSE,
                  xlab="",ylab="",zlab="",
-                 xlim=xlim, ylim=ylim, zlim=zlim,
+                 xlim=Xlim[,d[1]], ylim=Xlim[,d[2]], zlim=ylim,
                  ...)
         }
 
@@ -484,10 +394,10 @@ sectionview3d.km <- function(km_model, type = "UK",
                            conf_level = 0.95,
                            conf_fading = 0.5,
                            bg_fading = 1,
-                           mfrow = NULL,
+                           mfrow = c(1,1),
                            Xlab = NULL, ylab = NULL,
                            Xlim = NULL, ylim = NULL,
-                           title = NULL,
+                           title = NULL, title_sep = " | ",
                            add = FALSE,
                            engine3d = NULL,
                            ...) {
@@ -505,13 +415,13 @@ sectionview3d.km <- function(km_model, type = "UK",
         sdy_doe <- 0
     }
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
 
-    if (is.null(ylim)) {
+    if (is.null(ylim) && !add) {
         ymin <- min(y_doe-3*sdy_doe)
         ymax <- max(y_doe+3*sdy_doe)
         ylim <- c(ymin, ymax)
@@ -521,13 +431,8 @@ sectionview3d.km <- function(km_model, type = "UK",
     if (is.null(ylab)) ylab <- names(y_doe)
     if (is.null(Xlab)) Xlab <- names(X_doe)
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
-
-    if (is.null(conf_fading) || length(conf_fading) != length(conf_level)) {
+    if (is.null(conf_fading) ||
+        length(conf_fading) != length(conf_level)) {
         conf_fading <- rep(0.5/length(conf_level), length(conf_level))
     }
 
@@ -535,18 +440,18 @@ sectionview3d.km <- function(km_model, type = "UK",
     sectionview3d.function(fun = function(x) {
         DiceKriging::predict.km(km_model,type=type,newdata=x,checkNames=FALSE)$mean
     }, vectorized=TRUE,
-    dim = D, center = center,axis = axis,npoints = npoints,
+    center = center, axis = axis, mfrow = mfrow, Xlim = Xlim, ylim=ylim,
+    npoints = npoints,
     col_fun = col_fun, #conf_fading=conf_fading,
-    mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-    Xlim = rx, ylim=ylim, title = title, add = add, engine3d=engine3d, ...)
+    Xlab = Xlab, ylab = ylab,
+    title = title, title_sep  = title_sep, add = add, engine3d=engine3d, ...)
 
     if (!km_model@noise.flag)
         # plot design points
         sectionview3d.matrix(X = X_doe, y = y_doe,
-                       dim = D, center = center, axis = axis,
                        col_points = col_points,
                        col_fading_interval = conf_fading, bg_fading = bg_fading,
-                       mfrow = mfrow, Xlim = rx, ylim=ylim, add=TRUE)
+                       add=TRUE)
 
     # plot confidence bands
     for (l in conf_level) {
@@ -554,17 +459,16 @@ sectionview3d.km <- function(km_model, type = "UK",
                 p = DiceKriging::predict.km(km_model,type=type,newdata=x,checkNames=FALSE)
                 cbind(p$mean-qnorm(1-(1-l)/2) * p$sd, p$mean+qnorm(1-(1-l)/2) * p$sd)
             }, vectorized=TRUE,
-            dim = D, center = center,axis = axis,npoints = npoints,
+            npoints = npoints,
             col_fun = col_fun,
             col_fading_interval=conf_fading,
-            mfrow = mfrow, Xlim = rx, ylim=ylim, add = TRUE, engine3d=engine3d)
+            add = TRUE, engine3d=engine3d)
 
         if (km_model@noise.flag)
             sectionview3d.matrix(X = X_doe, y = cbind(y_doe-qnorm(1-(1-l)/2) * sdy_doe, y_doe+qnorm(1-(1-l)/2) * sdy_doe),
-                           dim = D, center = center, axis = axis,
                            col_points = col_points,
                            col_fading_interval = conf_fading, bg_fading = bg_fading,
-                           mfrow = mfrow, Xlim = rx, ylim=ylim, add=TRUE, engine3d=engine3d)
+                           add=TRUE, engine3d=engine3d)
     }
 
 }
@@ -583,10 +487,10 @@ sectionview3d_libKriging <- function(libKriging_model,
                            conf_level = 0.95,
                            conf_fading = 0.5,
                            bg_fading = 1,
-                           mfrow = NULL,
+                           mfrow = c(1,1),
                            Xlab = NULL, ylab = NULL,
                            Xlim = NULL, ylim = NULL,
-                           title = NULL,
+                           title = NULL, title_sep = " | ",
                            add = FALSE,
                            engine3d = NULL,
                            ...) {
@@ -605,13 +509,13 @@ sectionview3d_libKriging <- function(libKriging_model,
     } else
         stop(paste0("Kriging model of class ",class(libKriging_model)," is not yet supported."))
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
 
-    if (is.null(ylim)) {
+    if (is.null(ylim) && !add) {
         if (is.null(sdy_doe))
             ylim = range(y_doe)
         else {
@@ -627,12 +531,6 @@ sectionview3d_libKriging <- function(libKriging_model,
     if (is.null(Xlab)) Xlab <- names(X_doe)
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
 
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
-
     if (is.null(conf_fading) ||
         length(conf_fading) != length(conf_level))
         conf_fading <- rep(0.5/length(conf_level), length(conf_level))
@@ -640,19 +538,18 @@ sectionview3d_libKriging <- function(libKriging_model,
     sectionview3d.function(fun = function(x) {
             rlibkriging::predict(libKriging_model,x,return_stdev=FALSE)$mean
         }, vectorized=TRUE,
-        dim = D, center = center,axis = axis,npoints = npoints,
+        center = center, axis = axis, mfrow = mfrow, Xlim = Xlim, ylim=ylim,
+        npoints = npoints,
         col_fun = col_fun, #conf_fading=conf_fading,
-        mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-        Xlim = rx, ylim=ylim, title = title, add = add, engine3d=engine3d, ...)
+        Xlab = Xlab, ylab = ylab,
+        title = title, title_sep  = title_sep, add = add, engine3d=engine3d, ...)
 
     if (! inherits(libKriging_model, "NoiseKriging"))
         # plot design points
         sectionview3d.matrix(X = X_doe, y = y_doe,
-                       center = center, axis = axis,
                        col_points = col_points,
                        col_fading_interval = conf_fading, bg_fading = bg_fading,
-                       mfrow = mfrow,
-                       Xlim = rx, ylim=ylim, add=TRUE, engine3d=engine3d)
+                       add=TRUE, engine3d=engine3d)
 
     # plot confidence bands
     for (l in conf_level) {
@@ -660,17 +557,16 @@ sectionview3d_libKriging <- function(libKriging_model,
                 p = rlibkriging::predict(libKriging_model,x,return_stdev=TRUE)
                 cbind(p$mean-qnorm(1-(1-l)/2) * p$stdev, p$mean+qnorm(1-(1-l)/2) * p$stdev)
             }, vectorized=TRUE,
-            dim = D, center = center,axis = axis,npoints = npoints,
+            npoints = npoints,
             col_fun = col_fun,
             col_fading_interval=conf_fading,
-            mfrow = mfrow, Xlim = rx, ylim=ylim, add = TRUE, engine3d=engine3d)
+            add = TRUE, engine3d=engine3d)
 
         if (inherits(libKriging_model, "NoiseKriging")) # so sdy_doe=0
             sectionview3d.matrix(X = X_doe, y = cbind(y_doe-qnorm(1-(1-l)/2) * sdy_doe, y_doe+qnorm(1-(1-l)/2) * sdy_doe),
-                           center = center, axis = axis,
                            col_points = col_points,
                            col_fading_interval = conf_fading, bg_fading = bg_fading,
-                           mfrow = mfrow, Xlim = rx, ylim=ylim, add=TRUE, engine3d=engine3d)
+                           add=TRUE, engine3d=engine3d)
     }
 
 }
@@ -708,17 +604,17 @@ sectionview3d.Kriging <- function(Kriging_model,
                                    conf_level = 0.95,
                                    conf_fading = 0.5,
                                    bg_fading = 1,
-                                   mfrow = NULL,
+                                   mfrow = c(1,1),
                                    Xlab = NULL, ylab = NULL,
                                    Xlim = NULL, ylim = NULL,
-                                   title = NULL,
+                                   title = NULL, title_sep = " | ",
                                    add = FALSE,
                                    engine3d = NULL,
                                    ...) {
     sectionview3d_libKriging(Kriging_model,center,axis,npoints,
                              col_points=col_points,col_fun=col_fun,col=col,
                              conf_level,conf_fading,bg_fading,
-                             mfrow,Xlab, ylab,Xlim,ylim,title,add,engine3d,...)
+                             mfrow,Xlab, ylab,Xlim,ylim,title,title_sep,add,engine3d,...)
 }
 
 #' @param NuggetKriging_model an object of class \code{"Kriging"}.
@@ -754,17 +650,17 @@ sectionview3d.NuggetKriging <- function(NuggetKriging_model,
                                 conf_level = 0.95,
                                 conf_fading = 0.5,
                                 bg_fading = 1,
-                                mfrow = NULL,
+                                mfrow = c(1,1),
                                 Xlab = NULL, ylab = NULL,
                                 Xlim = NULL, ylim = NULL,
-                                title = NULL,
+                                title = NULL, title_sep = " | ",
                                 add = FALSE,
                                 engine3d = NULL,
                                 ...) {
     sectionview3d_libKriging(NuggetKriging_model,center,axis,npoints,
                              col_points=col_points,col_fun=col_fun,col=col,
                              conf_level,conf_fading,bg_fading,
-                             mfrow,Xlab, ylab,Xlim,ylim,title,add,engine3d,...)
+                             mfrow,Xlab, ylab,Xlim,ylim,title,title_sep,add,engine3d,...)
 }
 
 #' @param NoiseKriging_model an object of class \code{"Kriging"}.
@@ -800,17 +696,17 @@ sectionview3d.NoiseKriging <- function(NoiseKriging_model,
                                       conf_level = 0.95,
                                       conf_fading = 0.5,
                                       bg_fading = 1,
-                                      mfrow = NULL,
+                                      mfrow = c(1,1),
                                       Xlab = NULL, ylab = NULL,
                                       Xlim = NULL, ylim = NULL,
-                                      title = NULL,
+                                      title = NULL, title_sep = " | ",
                                       add = FALSE,
                                       engine3d = NULL,
                                       ...) {
     sectionview3d_libKriging(NoiseKriging_model,center,axis,npoints,
                              col_points=col_points,col_fun=col_fun,col=col,
                              conf_level,conf_fading,bg_fading,
-                             mfrow,Xlab, ylab,Xlim,ylim,title,add,engine3d,...)
+                             mfrow,Xlab, ylab,Xlim,ylim,title,title_sep,add,engine3d,...)
 }
 
 #' @param glm_model an object of class \code{"glm"}.
@@ -843,10 +739,10 @@ sectionview3d.glm <- function(glm_model,
                            conf_level = 0.95,
                            conf_fading = 0.5,
                            bg_fading = 1,
-                           mfrow = NULL,
+                           mfrow = c(1,1),
                            Xlab = NULL, ylab = NULL,
                            Xlim = NULL, ylim = NULL,
-                           title = NULL,
+                           title = NULL, title_sep = " | ",
                            add = FALSE,
                            engine3d = NULL,
                            ...) {
@@ -869,17 +765,11 @@ sectionview3d.glm <- function(glm_model,
     y_doe <- do.call(cbind,lapply(ylab,function(yn)glm_model$data[[yn]]))
     colnames(y_doe) <- ylab
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
-
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
 
     if (is.null(conf_fading) ||
         length(conf_fading) != length(conf_level))
@@ -892,17 +782,17 @@ sectionview3d.glm <- function(glm_model,
             colnames(x) <- Xlab
             predict.glm(glm_model, newdata=x, se.fit=FALSE)
         }, vectorized=TRUE,
-        dim = D, center = center,axis = axis, npoints = npoints,
+        center = center,axis = axis, mfrow = mfrow, Xlim = Xlim, ylim=range(y_doe),
+        npoints = npoints,
         col_fun = col_fun,
-        mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-        Xlim = rx, ylim=range(y_doe), title = title, add = add, engine3d=engine3d, ...)
+        Xlab = Xlab, ylab = ylab,
+        title = title, title_sep  = title_sep, add = add, engine3d=engine3d, ...)
 
     # plot design points
     sectionview3d.matrix(X = X_doe, y = y_doe,
-                         dim = D, center = center, axis = axis,
                          col_points = col_points,
                          col_fading_interval = conf_fading, bg_fading = bg_fading,
-                         mfrow = mfrow, Xlim = rx, ylim=range(y_doe), add=TRUE, engine3d=engine3d)
+                         add=TRUE, engine3d=engine3d)
 
     # plot confidence bands
     for (l in conf_level) {
@@ -912,10 +802,10 @@ sectionview3d.glm <- function(glm_model,
                 p = predict.glm(glm_model, newdata=x, se.fit=TRUE)
                 cbind(p$fit-qnorm(1-(1-l)/2) * p$se.fit, p$fit+qnorm(1-(1-l)/2) * p$se.fit)
             }, vectorized=TRUE,
-            dim = D, center = center,axis = axis,npoints = npoints,
+            npoints = npoints,
             col_fun = col_fun,
             col_fading_interval=conf_fading,
-            mfrow = mfrow, Xlim = rx, ylim=range(y_doe), add = TRUE)
+            add = TRUE)
     }
 }
 
@@ -949,10 +839,10 @@ sectionview3d.list <- function(modelFit_model,
                             col_fun = if (!is.null(col)) col else "blue",
                             col = NULL,
                             bg_fading = 1,
-                            mfrow = NULL,
+                            mfrow = c(1,1),
                             Xlab = NULL, ylab = NULL,
                             Xlim = NULL, ylim = NULL,
-                            title = NULL,
+                            title = NULL, title_sep = " | ",
                             add = FALSE,
                             engine3d = NULL,
                             ...) {
@@ -969,17 +859,11 @@ sectionview3d.list <- function(modelFit_model,
     if (is.null(Xlab)) Xlab <- names(X_doe)
     if (is.null(Xlab)) Xlab <- paste(sep = "", "X", 1:D)
 
-    ## find limits: rx is matrix with min in row 1 and max in row 2
-    rx <- apply(X_doe, 2, range)
-    if(!is.null(Xlim)) rx <- matrix(Xlim,nrow=2,ncol=D)
-    rownames(rx) <- c("min", "max")
-    drx <- unlist(rx["max", ]) - unlist(rx["min", ])
-
-    if (is.null(axis)) {
-        axis <- t(utils::combn(D, 2))
-    } else {
-        axis <- matrix(axis, ncol = 2)
-    }
+    ## find limits
+    if (is.null(Xlim) && !add)
+        Xlim <- apply(X_doe, 2, range)
+    else if (!is.null(Xlim))
+        Xlim <- matrix(Xlim,nrow=2,ncol=D)
 
     # plot mean
     sectionview3d.function(
@@ -988,16 +872,16 @@ sectionview3d.list <- function(modelFit_model,
             colnames(x) <- Xlab
             DiceEval::modelPredict(modelFit_model, x)
         }, vectorized=TRUE,
-        dim = D, center = center,axis = axis, npoints = npoints,
-    col_fun = col_fun, #conf_fading=conf_fading,
-        mfrow = mfrow, Xlab = Xlab, ylab = ylab,
-        Xlim = rx, ylim=range(y_doe), title = title, add = add, engine3d=engine3d, ...)
+        center = center,axis = axis,mfrow = mfrow,Xlim = Xlim, ylim=range(y_doe),
+        npoints = npoints,
+        col_fun = col_fun, #conf_fading=conf_fading,
+        Xlab = Xlab, ylab = ylab,
+        title = title, title_sep  = title_sep, add = add, engine3d=engine3d, ...)
 
     sectionview3d.matrix(X = X_doe, y = y_doe,
-                         dim = D, center = center, axis = axis,
                          col_points = col_points,
                          bg_fading = bg_fading,
-                         mfrow = mfrow, Xlim = rx, ylim=range(y_doe), engine3d=engine3d, add=TRUE)
+                         add=TRUE, engine3d=engine3d)
 }
 
 
@@ -1017,7 +901,7 @@ if(!isGeneric("sectionview3d")) {
 #' @export
 #' @examples
 #' ## A 2D example - Branin-Hoo function
-#' sectionview3d(branin, dim=2, col='black')
+#' sectionview3d(branin, col='black')
 #'
 #' \dontrun{
 #' ## a 16-points factorial design, and the corresponding response
@@ -1030,26 +914,26 @@ if(!isGeneric("sectionview3d")) {
 #' ## model: km
 #' model <- DiceKriging::km(design = design.fact, response = y)
 #' sectionview3d(model)
-#' sectionview3d(branin, dim=2, col='red', add=TRUE)
+#' sectionview3d(branin, col='red', add=TRUE)
 #' }
 #'
 #' if (requireNamespace("rlibkriging")) { library(rlibkriging)
 #' ## model: Kriging
 #' model <- Kriging(X = as.matrix(design.fact), y = as.matrix(y), kernel="matern3_2")
 #' sectionview3d(model)
-#' sectionview3d(branin, dim=2, col='red', add=TRUE)
+#' sectionview3d(branin, col='red', add=TRUE)
 #' }
 #'
 #' ## model: glm
 #' model <- glm(y ~ 1+ x1 + x2 + I(x1^2) + I(x2^2) + x1*x2, data=cbind(y,design.fact))
 #' sectionview3d(model)
-#' sectionview3d(branin, dim=2, col='red', add=TRUE)
+#' sectionview3d(branin, col='red', add=TRUE)
 #'
 #' if (requireNamespace("DiceEval")) { library(DiceEval)
 #' ## model: StepLinear
 #' model <- modelFit(design.fact, y, type = "StepLinear")
 #' sectionview3d(model)
-#' sectionview3d(branin, dim=2, col='red', add=TRUE)
+#' sectionview3d(branin, col='red', add=TRUE)
 #' }
 #' }
 #'
